﻿/*
 * imaeg - generic image utility in C#
 * Copyright (C) 2010  ed <tripflag@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License v2
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, refer to the following URL:
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;

namespace imaeg {
    public partial class Form1 : Form {
        public Form1() {
            InitializeComponent();
        }

        /*
         * F2       Rename
         * Alt-F2   Rename, goto next
         * num0-9   Move to subfolder
         * F11      Toggle fullscreen
         * F9       Gamma adjust
         * i        Image info
         */

        int cur, rot;           // Current image, rotation steps
        List<string> files;     // List of images to view
        string mvPre;           // Base folder for moving
        string path;            // Application path
        string initFile;        // Initial image to show
        bool folderInit;        // Have we made organizing folders yet?
        Rectangle rect;         // Widnow size memory (kiosk mode)
        Rectangle[] zoom;       // The zoom level memory
        bool lockEdges = true;  // When moving, lock onto screen edges?
        int scrollIdent;        // Sticky keys scrolling filter

        float tweekBrigh, tweekContr, tweekGamma;
        const float DEFAULT_BRIGH = 0f; // [-1..+1] where higher = brighter
        const float DEFAULT_CONTR = 1f; // [ 0..n ] where higher = stronger
        const float DEFAULT_GAMMA = 1f; // [ 0..n ] where higher = darker

        private void Form1_Load(object sender, EventArgs e) {
            this.Text += " v" + Application.ProductVersion;
            this.Show(); Application.DoEvents();
            rect = Rectangle.Empty;
            rot = 0; cur = 0;
            initFile = null;
            scrollIdent = 0;

            // Default gamma levels
            tweekBrigh = DEFAULT_BRIGH;
            tweekContr = DEFAULT_CONTR;
            tweekGamma = DEFAULT_GAMMA;
            zoom = new Rectangle[10];

            // Launched with an argument
            if (Program.args.Length > 0) {
                path = Program.args[0];
                if (System.IO.File.Exists(path)) {
                    initFile = path;
                    path = path.Substring(0,
                        path.LastIndexOf("\\"));
                }
                prepare(path);
            } else {
                // naw dude
                fileBrowser();
            }

            // Crude hack to start in fullscreen
            Form1_KeyDown(sender, new KeyEventArgs(Keys.F11));
            dsp.BackgroundImageLayout = ImageLayout.Tile;
        }

        private void Form1_DragOver(object sender, DragEventArgs e) {
            e.Effect = DragDropEffects.All;
        }

        private void Form1_DragDrop(object sender, DragEventArgs e) {
            prepare(((string[])e.Data.GetData(DataFormats.FileDrop))[0]);
        }

        private void dsp_Click(object sender, EventArgs e) {
            // This interferes with zooman, fuck it
            //fileBrowser();
        }

        void fileBrowser() {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.ShowDialog(); string fn = ofd.FileName;
            if (string.IsNullOrEmpty(fn)) return;
            fn = fn.Substring(0, fn.LastIndexOf("\\"));
            prepare(fn);
        }

        // Initialize application to given path
        void prepare(string path) {
            if (folderInit) {
                // App has already been prepared; method called
                // to delete empty/dummy organiizng folders
                for (int a = 0; a < 10; a++) {
                    try {
                        System.IO.Directory.Delete(mvPre + a);
                    } catch { }
                }
            }
            if (string.IsNullOrEmpty(path)) return;
            cur = 0;
            path += "\\";
            this.path = path;
            mvPre = path + "outdir";

            files = new List<string>(System.
                IO.Directory.GetFiles(path));
            for (int a = 0; a < files.Count; a++) {
                files[a] = files[a].Substring(path.Length);
                string ext = files[a].Substring(files[a].
                    LastIndexOf(".") + 1).ToLower();

                // Filter to images natively supported by .NET
                if (ext != "jpg" && ext != "png" &&
                    ext != "gif" && ext != "bmp" &&
                    ext != "emf" && ext != "ico" &&
                    ext != "wmf" && ext != "tiff" &&
                    ext != "jpeg") {
                    files.RemoveAt(a);
                    a--;
                }
            }
            files.Sort();

            // Are we to display a specific initial file?
            if (!string.IsNullOrEmpty(initFile)) {
                initFile = initFile.Substring(path.Length);
                for (int a = 0; a < files.Count; a++)
                    if (initFile == files[a])
                        cur = a;
            }
            nextPic(0);
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e) {
            //this.Text = e.KeyCode.ToString();
            if (e.KeyCode == Keys.R ||
                e.KeyCode == Keys.OemOpenBrackets ||
                e.KeyCode == Keys.OemCloseBrackets) {

                if (e.KeyCode == Keys.OemCloseBrackets) rot++;
                else if (e.KeyCode == Keys.OemOpenBrackets) rot--;
                else if (e.Shift) rot++; else rot--;
                if (rot > 3) rot = 0;
                if (rot < 0) rot = 3;
                nextPic(0);
            }

            if (e.KeyCode == Keys.Escape) Application.Exit();

            if (e.KeyCode == Keys.F1) if (DialogResult.OK == MessageBox.Show(
                "imaeg v" + Application.ProductVersion + " by ed <tripflag@gmail.com>\r\n\r\n" +
                "powered by #/G/TV ENTERPRISES, ltd.\r\n\r\n" +
                "Visit website?", "sup", MessageBoxButtons.OK))
                    System.Diagnostics.Process.Start(
                        "http://code.google.com/p/imaeg/");

            if (!e.Alt && !e.Control && !e.Shift && (
                e.KeyCode == Keys.Left ||
                e.KeyCode == Keys.PageUp ||
                e.KeyCode == Keys.Up)) nextPic(-1);

            if (!e.Alt && !e.Control && !e.Shift && (
                e.KeyCode == Keys.Right ||
                e.KeyCode == Keys.PageDown ||
                e.KeyCode == Keys.Down ||
                e.KeyCode == Keys.Space)) nextPic(1);

            if (e.KeyCode >= Keys.NumPad0 &&
                e.KeyCode <= Keys.NumPad9) {
                if (e.Alt) {

                    // Bright/cont/gamma ctl
                    if (e.KeyCode == Keys.NumPad1) tweekBrigh -= 0.03f;
                    if (e.KeyCode == Keys.NumPad2) tweekContr -= tweekContr / 12;
                    if (e.KeyCode == Keys.NumPad3) tweekGamma += tweekGamma / 12;
                    if (e.KeyCode == Keys.NumPad4) tweekBrigh = DEFAULT_BRIGH;
                    if (e.KeyCode == Keys.NumPad5) tweekContr = DEFAULT_CONTR;
                    if (e.KeyCode == Keys.NumPad6) tweekGamma = DEFAULT_GAMMA;
                    if (e.KeyCode == Keys.NumPad7) tweekBrigh += 0.03f;
                    if (e.KeyCode == Keys.NumPad8) tweekContr += tweekContr / 12;
                    if (e.KeyCode == Keys.NumPad9) tweekGamma -= tweekGamma / 12;
                    validateTweek();
                } else {

                    // Organization modo
                    int k = (int)e.KeyCode;
                    k -= (int)Keys.NumPad0;
                    if (!folderInit) {
                        if (DialogResult.Yes != MessageBox.Show(
                            "Enable organization modo?", "Faggotry",
                            MessageBoxButtons.YesNo)) return;
                        for (int a = 0; a < 10; a++) System.IO.
                            Directory.CreateDirectory(mvPre + a);
                        folderInit = true;
                    }
                    dsp.Image.Dispose();
                    dsp.Image = null;
                    System.IO.File.Move(path + files[cur],
                        mvPre + k + "\\" + files[cur]);
                    files.RemoveAt(cur);
                }
                nextPic(0);
            }
            if (e.KeyCode == Keys.F11) {
                if (this.rect == Rectangle.Empty) {
                    this.FormBorderStyle = FormBorderStyle.None;
                    this.WindowState = FormWindowState.Normal;
                    Application.DoEvents();
                    this.rect = this.Bounds;
                    this.Bounds = Screen.FromControl(this).Bounds;
                } else {
                    this.FormBorderStyle = FormBorderStyle.Sizable;
                    this.WindowState = FormWindowState.Normal;
                    Application.DoEvents();
                    this.Bounds = this.rect;
                    this.rect = Rectangle.Empty;
                }
            }
            if (e.KeyCode == Keys.I) {
                stat.Visible = !stat.Visible;
            }
            if (e.KeyCode == Keys.F2) {
                string fn = files[cur];
                string ext = fn.Substring(fn.LastIndexOf("."));
                fn = fn.Substring(0, fn.Length - ext.Length);
                string str = InputBox.Derp("Enter new filename", fn);
                if (str == null) return;
                fn = str + ext;
                if (files[cur] == fn) return;
                dsp.Image.Dispose();
                dsp.Image = null;
                stat.Text = "Moving...";
                Application.DoEvents();
                System.IO.File.Move(
                    path + files[cur],
                    path + fn);
                files[cur] = fn;
                nextPic(e.Alt ? 1 : 0);
            }
            if (e.KeyCode == Keys.F9) {
                try {
                    string b = InputBox.Derp(
                        "Please select viewer brightness.\n" +
                        "-100 to 100, higher is brighter.\n" +
                        "Default: " + (DEFAULT_BRIGH * 100),
                        (tweekBrigh * 100) + "");

                    string c = InputBox.Derp(
                        "Please select viewer contrast.\n" +
                        "0 to 9001, higher is stronger.\n" +
                        "Default: " + (DEFAULT_CONTR * 100),
                        (tweekContr * 100) + "");

                    string g = InputBox.Derp(
                        "Input viewer gamma.\n" +
                        "0 to 9001, higher is darker.\n" +
                        "Default: " + (DEFAULT_GAMMA * 100),
                        (tweekGamma * 100) + "");

                    if (b != null && c != null && g != null) {
                        tweekBrigh = (float)(Convert.ToDouble(b) / 100);
                        tweekContr = (float)(Convert.ToDouble(c) / 100);
                        tweekGamma = (float)(Convert.ToDouble(g) / 100);
                        validateTweek();
                        nextPic(0);
                    }
                } catch { }
            }
            if (e.KeyCode == Keys.Home) {
                dsp.Dock = DockStyle.Fill;
                int w = dsp.Width;
                int h = dsp.Height;
                dsp.Dock = DockStyle.None;
                dsp.Size = new Size(w, h);
                dsp.Location = Point.Empty;
                dsp.Dock = DockStyle.Fill;
            }
            if (e.KeyCode == Keys.Up) {
                if (e.Control) scrollSmooth(dsp.Left, dsp.Top + span.Height);
                    
                    // "Pseudocode"
                    //dsp.Top += span.Height;
                    /*while (from < to) {
                        if (from + 200 < to) {
                            from += 200;
                        } else if (from + 100 < to) {
                            from += 100;
                        } else if (from + 50 < to) {
                            from += 50;
                        } else if (from + 25 < to) {
                            from += 25;
                        } else if (from + 12 < to) {
                            from += 12;
                        } else if (from + 6 < to) {
                            from += 6;
                        } else if (from + 3 < to) {
                            from += 3;
                        } else from = to;
                        dsp.Top = from;
                        Application.DoEvents();
                        System.Threading.Thread.Sleep(50);
                    }*/
                
                if (e.Shift) scrollSmooth(dsp.Left, dsp.Top + span.Height / 2); //dsp.Top += span.Height / 2;
                validateBounds();
            }
            if (e.KeyCode == Keys.Down) {
                if (e.Control) scrollSmooth(dsp.Left, dsp.Top - span.Height); //dsp.Top -= span.Height;
                if (e.Shift) scrollSmooth(dsp.Left, dsp.Top - span.Height / 2); //dsp.Top -= span.Height / 2;
                validateBounds();
            }
            if (e.KeyCode == Keys.Left) {
                if (e.Control) scrollSmooth(dsp.Left + span.Width, dsp.Top); //dsp.Left += span.Width;
                if (e.Shift) scrollSmooth(dsp.Left + span.Width / 2, dsp.Top); //dsp.Left += span.Width / 2;
                validateBounds();
            }
            if (e.KeyCode == Keys.Right) {
                if (e.Control) scrollSmooth(dsp.Left - span.Width, dsp.Top); //dsp.Left -= span.Width;
                if (e.Shift) scrollSmooth(dsp.Left - span.Width / 2, dsp.Top); //dsp.Left -= span.Width / 2;
                validateBounds();
            }
            if (e.KeyCode == Keys.B) {
                dsp.BackgroundImage = null;
                this.BackColor = Color.Black;
                if (e.Control) this.BackColor = Color.White;
                if (e.Shift) dsp.BackgroundImage = new Bitmap(embedded("transparent.png"));
                if (e.Alt) using (ColorDialog cd = new ColorDialog()) {
                        cd.ShowDialog();
                        this.BackColor = cd.Color;
                    }
            }
            if (e.KeyCode == Keys.L) {
                lockEdges = !lockEdges;
            }
            if (e.KeyCode >= Keys.D0 &&
                e.KeyCode <= Keys.D9) {
                if (e.Shift) {
                    // Save this camera angle
                    zoom[e.KeyCode - Keys.D0] = dsp.Bounds;
                } else if (e.Alt) {
                    // Zoom to specific prefix
                    if (e.KeyCode == Keys.D1) {
                        // Scale 1:1
                        dsp.Dock = DockStyle.None;
                        dsp.Bounds = new Rectangle(Point.Empty, dsp.Image.Size);
                    }
                    if (e.KeyCode == Keys.D2) {
                        // Scale to Width
                        dsp.Dock = DockStyle.None;
                        double frac = span.Width / (1.0 * dsp.Image.Width);
                        int height = (int)(dsp.Image.Height * frac);
                        dsp.Bounds = new Rectangle(Point.Empty, new Size(span.Width, height));
                    }
                    if (e.KeyCode == Keys.D3) {
                        // Scale to Height
                        dsp.Dock = DockStyle.None;
                        double frac = span.Height / (1.0 * dsp.Image.Height);
                        int width = (int)(dsp.Image.Width * frac);
                        dsp.Bounds = new Rectangle(Point.Empty, new Size(width, span.Height));
                    }
                    if (e.KeyCode == Keys.D4) {
                        // Scale to screen (reduntant, "home")
                        dsp.Dock = DockStyle.Fill;
                        //scaleShrink();
                    }
                } else {
                    try {
                        // Recall camera angle (if any)
                        dsp.Dock = DockStyle.None;
                        dsp.Bounds = zoom[e.KeyCode - Keys.D0];
                    } catch { }
                }
            }
            /*if (e.Control &&
                e.KeyCode == Keys.S) {
                try {
                    dsp.Image.Save(files[cur] + ".png", ImageFormat.Png);
                    nextPic(1);
                } catch {
                    MessageBox.Show("Write failed");
                }
            }*/
        }

        System.IO.Stream embedded(string filename) {
            return System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("imaeg." + filename);
        }

        void nextPic(int dir) {
            cur += dir;
            if (cur >= files.Count) cur = 0;
            if (cur < 0) cur = files.Count - 1;
            dsp.Dock = DockStyle.Fill;

            stat.Text = "[!] #" + (cur + 1) + " " + files[cur];
            Application.DoEvents();
            if (dsp.Image != null) {
                dsp.Image.Dispose();
                dsp.Image = null;
            }
            try {
                Bitmap bm = new Bitmap(path + files[cur]);

                // Apply rotation
                if (rot == 1) bm.RotateFlip(RotateFlipType.Rotate90FlipNone);
                if (rot == 2) bm.RotateFlip(RotateFlipType.Rotate180FlipNone);
                if (rot == 3) bm.RotateFlip(RotateFlipType.Rotate270FlipNone);

                // Apply brightness/contrast/gamma
                if (tweekBrigh != DEFAULT_BRIGH ||
                    tweekContr != DEFAULT_CONTR ||
                    tweekGamma != DEFAULT_GAMMA) {
                    Bitmap tmp = bm;
                    bm = new Bitmap(tmp.Width, tmp.Height);
                    using (Graphics g = Graphics.FromImage(bm)) {
                        float b = tweekBrigh;
                        float c = tweekContr;
                        ImageAttributes derp = new ImageAttributes();
                        derp.SetColorMatrix(new ColorMatrix(new float[][]{
                            new float[]{c, 0, 0, 0, 0},
                            new float[]{0, c, 0, 0, 0},
                            new float[]{0, 0, c, 0, 0},
                            new float[]{0, 0, 0, 1, 0},
                            new float[]{b, b, b, 0, 1}}));
                        derp.SetGamma(tweekGamma);
                        g.DrawImage(tmp, new Rectangle(Point.Empty, bm.Size),
                            0, 0, bm.Width, bm.Height, GraphicsUnit.Pixel, derp);
                    }
                }
                dsp.Image = bm;
                stat.Text = "(" +
                    dsp.Image.Width + "x" +
                    dsp.Image.Height + ") " +
                    stat.Text.Substring(4);
            } catch (Exception ex) {
                dsp.Image = null;
                Application.DoEvents();
                //MessageBox.Show(ex.Source + " -> " +
                //    ex.Message + "\n" + ex.StackTrace);
                stat.Text = "[BAD] " + stat.Text.Substring(4);
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
            prepare(null);
        }

        void validateTweek() {
            tweekBrigh = (float)Math.Min(Math.Max(tweekBrigh, -0.99), 0.99);
            tweekContr = (float)Math.Min(Math.Max(tweekContr, 0.01), 10);
            tweekGamma = (float)Math.Min(Math.Max(tweekGamma, 0.01), 10);
        }

        Rectangle rc = Rectangle.Empty;
        private void dsp_MouseDown(object sender, MouseEventArgs e) {
            if (e.Button == MouseButtons.Middle) {
                if (rc.Width > 0) {
                    // Reset pan/zoom
                    rc = Rectangle.Empty;
                    dsp.Dock = DockStyle.Fill;
                } else {
                    // Top left corner
                    rc.Location = e.Location;
                }
            }
            loc = Cursor.Position;
            center = Cursor.Position;
            offX = e.X / (dsp.Width * 1.0);
            offY = e.Y / (dsp.Height * 1.0);
        }

        Point validateBounds() {
            return validateBounds(dsp.Bounds, true);
        }
        Point validateBounds(Rectangle reg, bool exec) {
            int x = reg.Left, y = reg.Top;
            if (lockEdges) {

                // Validate width
                if (reg.Width >= span.Width) {
                    // Larger than screen
                    if (x > 0) x = 0;
                    if (span.Width > reg.Width + x)
                        x = -(reg.Width - span.Width);
                } else {
                    // Smaller than screen
                    if (x < 0) x = 0;
                    if (span.Width < reg.Width + x)
                        x = span.Width - reg.Width;
                }

                // Validate height
                if (reg.Height >= span.Height) {
                    // Larger than screen
                    if (y > 0) y = 0;
                    if (span.Height > reg.Height + y)
                        y = -(reg.Height - span.Height);
                } else {
                    // Smaller than screen
                    if (y < 0) y = 0;
                    if (span.Height < reg.Height + y)
                        y = span.Height - reg.Height;
                }
            }
            //bool ret = (x == reg.Left && y == reg.Top);
            if (exec) dsp.Location = new Point(x, y);
            return new Point(x, y);
        }

        // Relative position reference
        double offX, offY, aspect;
        Point loc = Point.Empty;
        Point center = Point.Empty;

        bool doit = true;
        private void dsp_MouseMove(object sender, MouseEventArgs e) {
            if (!doit) return; doit = false;
            if (e.Button == MouseButtons.Right) {
                Point pos = loc;
                loc = Cursor.Position;
                aspect = dsp.Image.Width /
                    (1.0 * dsp.Image.Height);

                // Physically center if docked
                if (dsp.Dock != DockStyle.None)
                    scaleShrink();

                // Just to make things less complicated
                double w = dsp.Width; //, h = dsp.Height;
                int diffX = Cursor.Position.X - pos.X;
                int diffY = Cursor.Position.Y - pos.Y;

                // Pythagoras to find length
                double diff = Math.Sqrt(
                    Math.Pow(1.0 * diffX, 2) +
                    Math.Pow(1.0 * diffY, 2));

                // Retain most weighted sign
                if (Math.Abs(diffX) > Math.Abs(diffY))
                    diff *= Math.Sign(diffX);
                else diff *= Math.Sign(diffY);

                // Calculate new size and offset
                double newW = Math.Max(25,
                    w + w * 0.005 * diff);
                double newH = newW / aspect;
                double newOffX = -(newW * offX) + center.X;
                double newOffY = -(newH * offY) + center.Y;

                // Apply the change
                if (newW * newH < 104857600) { //10240^2
                    dsp.Width = (int)newW;
                    dsp.Height = (int)newH;

                    // Old pivot method; centered on screen
                    //dsp.Left -= (int)(dsp.Width - w) / 2;
                    //dsp.Top -= (int)(dsp.Height - h) / 2;

                    // New pivot method; weighted on original rightclick coordinate
                    dsp.Left = (int)newOffX;
                    dsp.Top = (int)newOffY;
                    Application.DoEvents();
                }

            } else if (e.Button == MouseButtons.Left) {
                Point pos = loc; loc = Cursor.Position;

                // Directly set location
                //dsp.Left += Cursor.Position.X - pos.X;
                //dsp.Top += Cursor.Position.Y - pos.Y;

                // Validate not out of bounds
                validateBounds(new Rectangle(
                    dsp.Left + (Cursor.Position.X - pos.X),
                    dsp.Top + (Cursor.Position.Y - pos.Y),
                    dsp.Width, dsp.Height), true);
            }
            doit = true;
        }

        void scrollSmooth(int x, int y) {
            
            // Filter repetitious scrolls
            scrollIdent++;
            int myIdent = scrollIdent;

            // Make sure we're within bounds
            Point xy = validateBounds(new Rectangle(x, y, dsp.Width, dsp.Height), false);
            x = xy.X; y = xy.Y;         // expand validated position
            
            int xa = dsp.Left;          // read current ("from") pos
            int ya = dsp.Top;
            int xd = x - xa;            // difference between cur and target
            int yd = y - ya;
            int xs = Math.Sign(xd);     // get direction
            int ys = Math.Sign(yd);
            xd *= xs;                   // filter direction from distance
            yd *= ys;
            Size sz = dsp.Size;
            int[] stepsize = { 200, 100, 50, 25, 12, 6, 3, 2, 1 };
            while (xd > 1 || yd > 1) {
                bool xb = false, yb = false;            // stepped this axis yet?
                foreach (int step in stepsize) {        // try all stepsizes
                    Point newpos = new Point(xa, ya);   // original position
                    if (!xb && step < xd) {             // can & should step X axis?
                        xd -= step;                     // subtract steps from dist
                        xa += step * (1 * xs);         // add step to position
                        xb = true;                      // stepped X axis
                    }
                    if (!yb && step < yd) {
                        yd -= step;
                        ya += step * (1 * ys);
                        yb = true;
                    }
                    /*if (step < dist) {
                        dist -= step;
                        from += step * (-1 * sign);
                        if (!validateBounds(new Rectangle(dsp.Left, from, dsp.Width, dsp.Height))) dist = 0;
                        Application.DoEvents();
                        System.Threading.Thread.Sleep(50);
                        break;
                    }*/
                }
                if (myIdent != scrollIdent) break;
                validateBounds(new Rectangle(xa, ya, sz.Width, sz.Height), true);
                Application.DoEvents();
                System.Threading.Thread.Sleep(20);
            }
        }

        void scaleShrink() {
            int durrX, durrY;
            double derpX = dsp.Image.Width / (1.0 * dsp.Width);
            double derpY = dsp.Image.Height / (1.0 * dsp.Height);
            if (derpX > derpY) {
                durrX = dsp.Width;
                durrY = (int)(dsp.Image.Height / derpX);
            } else {
                durrY = dsp.Height;
                durrX = (int)(dsp.Image.Width / derpY);
            }
            dsp.Dock = DockStyle.None;
            dsp.Location = Point.Empty;
            dsp.Size = new Size(durrX, durrY);
        }
    }
}